package recommendation.main;


import org.neo4j.cypher.javacompat.ExecutionEngine;
import org.neo4j.cypher.javacompat.ExecutionResult;
import org.neo4j.graphalgo.GraphAlgoFactory;
import org.neo4j.graphdb.DynamicLabel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.DynamicRelationshipType;
import org.neo4j.graphdb.GraphDatabaseService;
import org.neo4j.graphdb.Label;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Path;
import org.neo4j.graphdb.Relationship;
import org.neo4j.graphdb.RelationshipType;
import org.neo4j.graphdb.Transaction;
import org.neo4j.graphdb.factory.GraphDatabaseFactory;
import org.neo4j.graphdb.factory.GraphDatabaseSettings;
import org.neo4j.graphdb.index.Index;
import org.neo4j.graphdb.index.IndexHits;
import org.neo4j.graphdb.index.ReadableIndex;
import org.neo4j.graphdb.traversal.TraversalDescription;

import weka.clusterers.SimpleKMeans;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.neighboursearch.*;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.AddCluster;










//import org.neo4j.kernel.impl.traversal.TraversalDescriptionImpl;
import org.neo4j.kernel.impl.util.FileUtils;

public class Neo4jclass {

	public GraphDatabaseService graphDB;
	public Index<Node> indexPerformer;
	public Index<Node> indexSong ;
	public Index<Node> indexAlbum ;
	public Index<Node> indexLog;
	public Index<Node> indexGenre;
	public Index<Relationship> indexLogExist;
	
	public ArrayList<String> songs = new ArrayList<String>();
	public ArrayList<Integer> songArray = new ArrayList<Integer>();
	public ArrayList<frequency> frequencyList = new ArrayList<frequency>();
	
	private static enum RelTypes4 implements RelationshipType {
		LISTEN
	}
	public String argedorDataFolderPath; // log source
	public String workspacePath; // dbnin olusacagi yer

	public void setWorkspace(String workspacePath) 
	{
		
		this.workspacePath =  workspacePath;

		
	}
	public void start()
	{
		
		createDB();
		//createSong();
		//createPerformer();
		//createAlbum();
		//createRelation();
		//createLog();	
	}
	public void startAtBeginning(String argedorDataFolderPath,String workspacePath)//14Mayis Eklendi kontrol edilmeli
	{
		this.argedorDataFolderPath=argedorDataFolderPath;
		this.workspacePath =  workspacePath;
		createDB();
		createSong();
		createPerformer();
		createAlbum();
		createRelation();
		createLog();	
	}//14Mayis Eklendi
	void createDB()
	{
		
		graphDB = (GraphDatabaseService)new GraphDatabaseFactory().newEmbeddedDatabaseBuilder( workspacePath).//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\db\\graphDB" ).
                setConfig( GraphDatabaseSettings.node_keys_indexable, "name" ).
                setConfig( GraphDatabaseSettings.relationship_keys_indexable, "name" ).
                setConfig( GraphDatabaseSettings.node_auto_indexing, "true" ).
                setConfig( GraphDatabaseSettings.relationship_auto_indexing, "true" ).
                newGraphDatabase();
		
		Transaction tx = graphDB.beginTx();
		indexPerformer = this.graphDB.index().forNodes("Performer");
		indexSong = this.graphDB.index().forNodes("Song");
		indexAlbum = this.graphDB.index().forNodes("Album");
		indexLog = this.graphDB.index().forNodes("User");
		indexGenre = this.graphDB.index().forNodes("Song");
		indexLogExist = this.graphDB.index().forRelationships("Listening");
		tx.success();
		tx.close();
		registerShutdownHook(graphDB);
	}

	void createPerformer()
	{
		Transaction tx = graphDB.beginTx();
		Label label = DynamicLabel.label("Performer");
		try
		{
			try
			{
				String line = null;
				int count = 0;
				BufferedReader br = new BufferedReader(new FileReader(argedorDataFolderPath+ "\\performer.txt"));//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\data\\performer.txt"));
				try {
					while((line = br.readLine())!= null)
					{
						count += 1;
						if(count != 1)
						{
							String [] parts = line.split("\t");
							Node node1 = graphDB.createNode();
							if(parts.length > 0)
							{
								node1.setProperty("ID", parts[0]);
							}
							else
							{
								node1.setProperty("ID", "NULL");
							}
							if(parts.length > 1)
							{
								node1.setProperty("PROVIDERID",parts[1]);
							}
							else
							{
								if(parts.length > 1)
								{
										//System.out.println(parts[0]);
								}
								node1.setProperty("PROVIDERID","NULL");
							}
							if(parts.length > 2)
							{
								node1.setProperty("NAME", parts[2]);
							}
							else
							{
								if(parts.length > 1)
								{
									//System.out.println(parts[0]);
								}
								node1.setProperty("NAME", "NULL");
							}
							indexPerformer.add(node1, "ID", parts[0]);
							node1.addLabel(label);
							if((count > 1) && (count%1000 == 0))
							{
								tx.success();
								tx.close();
								tx = graphDB.beginTx();
							}
						}
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				try {
					br.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			tx.success();
			//System.out.println("performer bitti");
		}
		finally
		{
			tx.close();
			
		}
		//-----------

	}

	void createSong()
	{
		Transaction tx = graphDB.beginTx();
		Label label = DynamicLabel.label("Song");
		try
		{
			try
			{
				String line = null;
				int count = 0;
				BufferedReader br = new BufferedReader(new FileReader(argedorDataFolderPath + "\\song.txt" ));//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\data\\song.txt"));
				try {
					while((line = br.readLine())!= null)
					{
						count += 1;
						if (count != 1) {
							String[] parts = line.split("\t");
							Node node1 = graphDB.createNode();
							node1.setProperty("ID", parts[0]);
							node1.setProperty("PROVIDERID", parts[1]);
							node1.setProperty("GENREID", parts[2]);
							node1.setProperty("CREATETIME", parts[3]);
							node1.setProperty("ALBUMID", "NULL");
							node1.setProperty("PERFORMERID", "NULL");

							if (parts.length > 4) {
								node1.setProperty("NAME", parts[4]);
							} else {
								node1.setProperty("NAME", "???");
							}

							node1.setProperty("ALLOWEDWEBSTREAM", parts[5]);
							node1.setProperty("ALLOWEDWEBDOWNLOAD", parts[6]);
							node1.setProperty("ALLOWEDWEBVIDEOSTREAM", parts[7]);
							node1.setProperty("ALLOWEDMOBILESTREAM", parts[8]);
							node1.setProperty("ALLOWEDWAPSTREAM", parts[9]);

							indexSong.add(node1, "ID", parts[0]);
							indexGenre.add(node1, "Genre", parts[2]);
							node1.addLabel(label);
							if((count > 1) && (count%1000 == 0))
							{
								tx.success();
								tx.close();
								tx = graphDB.beginTx();
							}
						}
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				try {
					br.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			tx.success();
			//System.out.println("song bitti");
		}
		finally
		{
			tx.close();
		}
		//-----------

	}
	
	void createAlbum()
	{
		Transaction tx = graphDB.beginTx();
		Label label = DynamicLabel.label("Album");
		try
		{
			try
			{
				String line = null;
				int count = 0;
				BufferedReader br = new BufferedReader(new FileReader(argedorDataFolderPath + "\\album.txt"));//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\data\\album.txt"));
				try {
					while((line = br.readLine())!= null)
					{
						count += 1;
						if (count != 1) {
							String[] parts = line.split("\t");
							Node node1 = graphDB.createNode();
							node1.setProperty("ID", parts[0]);
							node1.setProperty("PROVIDERID", parts[1]);
							node1.setProperty("CREATETIME", parts[2]);
							node1.setProperty("ALBUMNAME", parts[3]);
							indexAlbum.add(node1, "ID", parts[0]);

							node1.addLabel(label);
							if((count > 1) && (count%1000 == 0))
							{
								tx.success();
								tx.close();
								tx = graphDB.beginTx();
							}
						}
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				try {
					br.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			tx.success();
			//System.out.println("album bitti");
		}
		finally
		{
			tx.close();
		}
		//-----------

	}
	
	void createRelation()
	{
		
		Transaction tx = graphDB.beginTx();
		Label label = DynamicLabel.label("Song");
		try
		{
			try
			{
				String line = null;
				int count = 0;
				BufferedReader br = new BufferedReader(new FileReader(argedorDataFolderPath + "\\map.txt"));//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\data\\map.txt"));
				try {
					while((line = br.readLine())!= null)
					{
						count += 1;
						if (count != 1) {
							String[] parts = line.split("\t");
							Node songNode = (Node) indexSong.get("ID", parts[2]).getSingle();
							if(songNode == null)
							{
								songNode = graphDB.createNode();
								songNode.setProperty("ID", parts[2]);
								songNode.setProperty("PROVIDERID", "NULL");
								songNode.setProperty("GENREID", "NULL");
								songNode.setProperty("CREATETIME", "NULL");
								songNode.addLabel(label);
							}
							songNode.setProperty("ALBUMID", parts[1]);
							songNode.setProperty("PERFORMERID", parts[0]);
							
							if((count > 1) && (count%1000 == 0))
							{
								tx.success();
								tx.close();
								tx = graphDB.beginTx();
							}

						}
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				try {
					br.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			tx.success();
			//System.out.println("relation bitti");
		}
		finally
		{
			tx.close();
		}
		//-----------
	}
	
	void createLog()
	{
		int count,i, folderNum;;
		Transaction tx;
		Label label = DynamicLabel.label("User");
		BufferedReader br;
	//	for(folderNum = 50 ; folderNum < 54 ; folderNum = folderNum + 2)
	//	{
			File folder = new File(argedorDataFolderPath);//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\data\\"+ folderNum);
			File[] listOfFiles = folder.listFiles();
			for(i = 0; i < listOfFiles.length; i++)
			{
				File file = listOfFiles[i];
				//System.out.println(file.getAbsoluteFile());
				if( file.isFile() && !(file.getName().endsWith(".txt")))
				{
					count = 0;
					tx = graphDB.beginTx();
					//System.out.println("yeni log basladi: "+ i);
					try
					{
						try
						{ 
								String line = null;
								br = new BufferedReader(new FileReader(file.getAbsoluteFile()));
								try {
									//System.out.println("log basladi");
									while((line = br.readLine())!= null)
									{
											count+=1;
											String[] parts = line.split(",");
											Node node1;
											Node controlNode = (Node) indexLog.get("userid", parts[0]).getSingle();
											
											if(controlNode == null)
											{
												node1 = graphDB.createNode();
												node1.setProperty("userid", parts[0]);
												indexLog.add(node1, "userid", parts[0]);
												node1.addLabel(label);
											}
											else
											{
												node1 = controlNode;
											}
											
											Node songNode = (Node) indexSong.get("ID", parts[1]).getSingle();
											//System.out.println(performerNode.getProperty("NAME"));
				
											Relationship relationship; 
											
											if(songNode != null)
											{
												IndexHits<Relationship> rel = indexLogExist.get("User_Song", parts[0] + "+" + parts[1]); 
												SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
												Date d1 = null;
												Date d3 = null; // current time with correct format
												Date currentTime = new Date(); // current time with incorrect format
												String currentTimeString = format.format(currentTime);
												try 
												{
													d3 = format.parse(currentTimeString);
												} 
												catch (ParseException e1)
												{
													e1.printStackTrace();
												}
												try 
												{
													d1 = format.parse(parts[4]);
												} catch (ParseException e) {
													
												}
												if(rel.size() == 0)
												{
													int number = 1;
													double freq;
													long timediff = d3.getTime() - d1.getTime();
													double mindiff = ((double)timediff)/((double)(60*1000)); //calculate frequency over minute
													freq = ((double)number)/mindiff;
													relationship = node1.createRelationshipTo(songNode,
															RelTypes4.LISTEN);
													relationship.setProperty("timeofaction", parts[4]);
													relationship.setProperty("ratingValue", Double.parseDouble(parts[5]));
													relationship.setProperty("channel", parts[6]);
													relationship.setProperty("numberofaction", number);
													relationship.setProperty("first_timeofaction", parts[4]);
													relationship.setProperty("frequency", freq);
													indexLogExist.add(relationship, "User_Song", parts[0]+ "+" + parts[1]);
												}
												else
												{
													relationship = rel.getSingle();
													//System.out.println(parts[0]);
													long timediff;
													double freq;
													int number = (int) relationship.getProperty("numberofaction");
													number = number + 1; // increase the number of the action
													String date = (String) relationship.getProperty("first_timeofaction");
													Date d2 = null;
													try
													{
														d2 = format.parse(date);
													} 
													catch (ParseException e) 
													{
														
													}
													timediff = d3.getTime() - d2.getTime();
													double mindiff = ((double)timediff)/((double)(60*1000)); //calculate frequency over minute
													freq = ((double)number)/mindiff;
													relationship.setProperty("numberofaction", number);
													relationship.setProperty("timeofaction", parts[4]);
													relationship.setProperty("ratingValue", Double.parseDouble(parts[5]));
													relationship.setProperty("channel", parts[6]);
													relationship.setProperty("frequency", freq);
												}
											}
											
											if((count > 1) && (count%1000 == 0))
											{
												tx.success();
												tx.close();
												tx = graphDB.beginTx();
											}
										}
									
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
				
								try {
									br.close();
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
							catch (FileNotFoundException e)
							{
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						
			
						tx.success();
						//System.out.println("log bitti");
					
					}
					finally
					{
						tx.close();
						
					//-----------
					}
				}
			}
	//}
		
		
		
	}
	
	

public Instances initializeWekaInstances() {  
		
        Instances instance = null;   
        // 6 olacak num of attr 
        int numberOfAttributes = 1;  
        FastVector attributes = new FastVector();  
        /** 
        * Set all attributes like Weka Arff file 
        * @attribute Att0 
        * @attribute Att1 
        * @attribute Att2 
        */  
        for (int i = 0; i < numberOfAttributes; i++) {  
            attributes.addElement(new Attribute("Att" + i));  
        }  
        instance = new Instances("WekaClusteringDataSet", attributes, 0);  
	    
	    return instance; 
		
	}
	
	public Instance convertToWekaIntance(Instances instances, Map<String, Object> row, List<String> columns){  
		  
		  //  ResultSetMetaData md = result.getMetaData();  
		    // number of the coloumn for an intance  
		//	Node n4;
			//Node n3;
			String namep, namea;
			Double timeofAct;
			double freq;
		    int attributeCount = 1;
		    int numofact = 0;
		    Instance wekaInstance = new Instance(attributeCount);  
		    wekaInstance.setDataset(instances);
		    
		    int count3=0;
			
		    int check=0;

		    	for (String column : columns) {
		    		if(check == 0) 
		    		{
		    			//System.out.println("burasi neymiss :" + row.get(column).toString());
		    			double rating= Double.parseDouble(row.get(column).toString());
		    			//System.out.println(rating);
						//wekaInstance.setValue(check, rating );
						wekaInstance.setWeight(rating);
		    		}
		    		
		    		if(check==5)
		    		{
		    			// 0:id 1:songName 2:albumid 3:performerid 4:timeofaction 5:frequecny 6:numberofaction 7:ratingvalue
		    			double rating= Double.parseDouble(row.get(column).toString());
		    			//System.out.println(rating);
						wekaInstance.setValue(check-5, rating );
		    		}
		    		check++;
		    	}
			
		    return wekaInstance;  
	}
	
	public void mostFrequent()
	{
		for (int i = 0; i < songArray.size(); i++) 
        {     
                int freqa = 1;  
                if(i<=songArray.size()-2)
                {
                    while(songArray.get(i)==songArray.get(i+1))
                    {
                        freqa++;
                        i++;
                        if(i==songArray.size()-1)
                        {
                            break;
                        }
                    }
                    frequency f = new frequency(songArray.get(i),freqa);
                    frequencyList.add(f);
                }
                else
                {
                    frequency f = new frequency(songArray.get(i),freqa);
                    frequencyList.add(f);
                }
         }
         Collections.sort(frequencyList);
	}
	public void findNeigbor(String ID, ArrayList<RecommendationList> recList) throws Exception
	{
		
//		COLLABORATIVE
		int count = 0;
		int count2 = 0;
		int check = 0;
		Node n3;
		Node n4;
		String similaruser = "";
		String namea = null;
		String names = null;
		String namep = null;
		RecommendationList rl;
		ExecutionEngine engine = new ExecutionEngine(graphDB);
		Transaction tx = graphDB.beginTx();
		ExecutionResult result;
		ExecutionResult result2;
		ExecutionResult result3;
		ExecutionResult result4;
		
		result = engine.execute("START me = node:User(userid='" + ID + "') MATCH (me)-[myrate:LISTEN]-> (song) <-[otherrate:LISTEN]-(similaruser) WHERE ((myrate.ratingValue > 0.6) AND (otherrate.ratingValue > 0.6) AND (myrate.numberofaction > 1) AND (ABS(myrate.frequency - otherrate.frequency) <= ((myrate.frequency + otherrate.frequency)/10))) RETURN similaruser, count(*)ORDER BY count(*)DESC Limit 5");
//		if(result == null)
//		{
//			result = engine.execute("START me = node:User(userid='" + ID + "') MATCH (me)-[myrate:LISTEN]-> (song) <-[otherrate:LISTEN]-(similaruser) WHERE (ABS(myrate.ratingValue -otherrate.ratingValue) < 0.2) AND (ABS(myrate.frequency - otherrate.frequency) <= ((myrate.frequency + otherrate.frequency)/10)) RETURN similaruser, count(*)ORDER BY count(*)DESC Limit 5");
//			System.out.println("ortak sarki dinlemis kullanici yok");
//		}
		
		ArrayList<String> recsongsid = new ArrayList<String>(); //recommend listesinde olani tekrar listeye almamak icin buradan kontrol

		for(Map<String, Object> row: result)
		{
			check = 0;
			for(Entry<String,Object> column:row.entrySet())
			{
				if(check == 0)
				{
					count++;
					if(count != 1)
					{
						similaruser += ",";
					}
					String[] parts = column.getValue().toString().split("\\[");
					String[] parts2 = parts[1].split("\\]");
					similaruser += parts2[0]; 
					//similaruser += column.getValue().toString();
					//System.out.println(similaruser);
				}
				check++;
				
			}
		}
		//System.out.println("Bu bos mu " + similaruser);
		if(similaruser.equals(""))
		{
//			result = engine.execute("START me = node:User(userid='" + ID + "') MATCH (me)-[myrate:LISTEN]-> (song) <-[otherrate:LISTEN]-(similaruser) RETURN similaruser, count(*)ORDER BY count(*)DESC Limit 5");
//			for(Map<String, Object> row: result)
//			{
//				check = 0;
//				for(Entry<String,Object> column:row.entrySet())
//				{
//					if(check == 0)
//					{
//						count++;
//						if(count != 1)
//						{
//							similaruser += ",";
//						}
//						String[] parts = column.getValue().toString().split("\\[");
//						String[] parts2 = parts[1].split("\\]");
//						similaruser += parts2[0]; 
//						//similaruser += column.getValue().toString();
//						//System.out.println(similaruser);
//					}
//					check++;
//					
//				}
//			}
			//System.out.println("COLD START");
			result3 = engine.execute("START me = node:User(userid= '" + ID + "') MATCH (me) - [myrate:LISTEN] -> (song) RETURN song.GENREID, count(*) ORDER BY count(*)DESC LIMIT 1");
			String maxgenre = null;
			int colcount = 0;
			for(Map<String, Object> row: result3)
			{
				List<String> columns = result3.columns();
				for(String column : columns)
				{
					if(colcount == 0)
					{
						maxgenre = row.get(column).toString();	
						//System.out.println("max genre: " + maxgenre);
					}
					colcount++;
				}
			}
			
			result4 = engine.execute("START u=node:Song(\"*:*\") MATCH (a)-[rel:LISTEN]->(u) WHERE u.GENREID = '" + maxgenre + "'RETURN  u.NAME, u.ALBUMID, u.PERFORMERID, u.ID, count(*) ORDER BY count(*) DESC LIMIT 5");
			
			for(Map<String, Object> row2: result4)
			{
				int itr = 0;
				String namesong = null, idalbum = null, idperformer = null, songid=null;
				List<String> columns = result4.columns();
				
				for (String column : columns)
				{
					
					if(itr == 0)
					{
						namesong = row2.get(column).toString();		
					}
					else if(itr == 1)
					{
						idalbum = row2.get(column).toString();		
					}
					else if(itr == 2)
					{
						idperformer = row2.get(column).toString();	
					}
					else if(itr == 3) //YENI!!!
					{
						songid = row2.get(column).toString();	
					}
					itr++;
				}
				
				
				Node node2 = (Node) indexAlbum.get("ID", idalbum).getSingle();
				String albumName = (String) node2.getProperty("ALBUMNAME");
				
				Node node3 = (Node) indexPerformer.get("ID", idperformer).getSingle();
				String perfName = (String) node3.getProperty("NAME");

				rl = new RecommendationList(albumName,namesong,perfName,songid);
				recList.add(rl);
			}
			tx.success();
			tx.close();
			return;
		}
		
	    
	    KDTree myKDTree; 
	    Instance output;
	    Instances out2;
	    Instances res = null;
	    
	    Instances dataSource = null;
		result = engine.execute("START me = node:User(userid='" + ID + "') MATCH (me) - [myrate:LISTEN] -> (song) WHERE ((myrate.ratingValue > 0.6) AND (myrate.numberofaction > 1)) RETURN distinct(song.ID),song.NAME, song.ALBUMID, song.PERFORMERID, myrate.timeofaction, myrate.frequency, myrate.numberofaction, myrate.ratingValue ORDER BY myrate.frequency LIMIT 5");
		result2 = engine.execute("START me=node:User(userid='" + ID + "'), similarusers = node(" + similaruser +") MATCH (similarusers)-[ot:LISTEN]->(song2),(similarusers)-[other:LISTEN]->(song)<-[mylisten:LISTEN]-(me) WHERE NOT(song2.ID =  song.ID) RETURN distinct(song2.ID),song2.NAME, song2.ALBUMID, song2.PERFORMERID, ot.timeofaction, ot.frequency, ot.numberofaction, ot.ratingValue");
	        //ResultSet resultSet = null; // you must get data from database as Resultset  
	        dataSource = initializeWekaInstances();  
	        res = initializeWekaInstances();
	        

			for(Map<String, Object> row2: result2)
			{
				List<String> columns = result2.columns();
				Instance tempIns = convertToWekaIntance(dataSource, row2, columns);
				dataSource.add(tempIns);
			}
	        myKDTree = new KDTree(); 
	        //System.out.println("build kd-tree"); 
	        try 
	        {
	        	myKDTree.setInstances(dataSource);
	        }
	        catch(Exception e) 
	        {
	        	e.printStackTrace();
	        }
	        ArrayList<String> albumIds = new ArrayList<String>();
	        ArrayList<String> performerIds = new ArrayList<String>();
	        ArrayList<String> genreIds = new ArrayList<String>(); // NEW
			for(Map<String, Object> row: result)
			{
				List<String> columns = result.columns();
				Instance tempIns = convertToWekaIntance(dataSource, row, columns);
				
				//res.kthSmallestValue((int) res.instance(i).value(1), (19+i));
				int idd = (int)tempIns.weight();
				String idsong = Integer.toString(idd);
				recsongsid.add(idsong);
				//result = engine.execute("START me = node:Song(ID='" + idsong + "') MATCH (me) - [myrate:LISTEN] -> (song) WHERE ((myrate.ratingValue > 0.6) AND (myrate.numberofaction > 1)) RETURN distinct(song.ID),song.NAME, song.ALBUMID, song.PERFORMERID, myrate.timeofaction, myrate.frequency, myrate.numberofaction, myrate.ratingValue ORDER BY myrate.frequency LIMIT 5");
				Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
				String songName = (String) node1.getProperty("NAME");
				//System.out.println(songName);
				Node node2 = (Node) indexAlbum.get("ID", node1.getProperty("ALBUMID").toString()).getSingle();
				albumIds.add(node1.getProperty("ALBUMID").toString());
				
				Node node3 = (Node) indexPerformer.get("ID", node1.getProperty("PERFORMERID").toString()).getSingle();
				performerIds.add(node1.getProperty("PERFORMERID").toString());
				genreIds.add(node1.getProperty("GENREID").toString()); // NEW
				String albumName = (String) node2.getProperty("ALBUMNAME");
				String perfName = (String) node3.getProperty("NAME");

				//output = myKDTree.nearestNeighbour(tempIns);
				//res.add(output);
				
				
				//System.out.println(songName + " " + perfName + " " + albumName);
				
				out2 = myKDTree.kNearestNeighbours(tempIns, 5);
				for(int i = 0; i < out2.numInstances(); i++)
				{
					//System.out.println("lalalallalala " + out2.instance(i).value(0));
					res.add(out2.instance(i));
				}
				
				
			}
			
			double max = 0;
			int index = 0;
			int countrec = 0;
			
			for(int i=0;i<res.numInstances();i++)  //NEW
			{
				int idd = (int)res.instance(index).weight();
				String idsong = Integer.toString(idd);
				songArray.add(idd);
			}
			mostFrequent();
			// NEW
			for(int y = 0; y < frequencyList.size() ; y++)
			{
				if(frequencyList.get(y).freq < 3)
				{
					break;
				}
				for(int i = 0; i < res.numInstances(); i++)
				{
					int idd = (int)res.instance(i).weight(); //index degisti!!!!
					String idsong = Integer.toString(idd);
					Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
					if((idd == frequencyList.get(y).id) && (countrec != 5) && !(recsongsid.contains(idsong)))
					{
						double zero = 0;

						res.instance(i).setValue(0, zero);
						countrec ++;

						recsongsid.add(idsong);
						songs.add(idsong);
					}
				}
			}
			for(int i = 0; i < res.numInstances(); i++)
			{
				int idd = (int)res.instance(i).weight();
				String idsong = Integer.toString(idd);
				Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
				
				if(res.instance(i).value(0) != 0) // distaki for loopta listeye alinmis olabilir
				{
					
					if(albumIds.contains((node1.getProperty("ALBUMID").toString())) && countrec != 5  && !(recsongsid.contains(idsong)))
					{
						double zero = 0;
						//max = 0;
						res.instance(i).setValue(0, zero);
						countrec ++;
						//Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
						String songName = (String) node1.getProperty("NAME");
						
						Node node2 = (Node) indexAlbum.get("ID", node1.getProperty("ALBUMID").toString()).getSingle();
						Node node3 = (Node) indexPerformer.get("ID", node1.getProperty("PERFORMERID").toString()).getSingle();
						
						String albumName;
						String perfName;
						if(node2.getProperty("ALBUMNAME") != null)
						{
							albumName = (String) node2.getProperty("ALBUMNAME");
						}
						else
						{
							albumName = "Unknown Album";
						}
						if(node3.getProperty("NAME") != null)
						{
							perfName = (String) node3.getProperty("NAME");
						}
						else
						{
							perfName = "Unknown Performer"; 
						}
						recsongsid.add(idsong);
						songs.add(idsong);
						
						rl = new RecommendationList(albumName,songName,perfName,idsong);
						recList.add(rl);
					}
					else if(performerIds.contains((node1.getProperty("PERFORMERID").toString())) && countrec != 5 && !(recsongsid.contains(idsong)))
					{
						double zero = 0;
						//max = 0;
						res.instance(i).setValue(0, zero);
						countrec ++;
						//Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
						String songName = (String) node1.getProperty("NAME");
						
						Node node2 = (Node) indexAlbum.get("ID", node1.getProperty("ALBUMID").toString()).getSingle();
						Node node3 = (Node) indexPerformer.get("ID", node1.getProperty("PERFORMERID").toString()).getSingle();
						
						String albumName;
						String perfName;
						if(node2.getProperty("ALBUMNAME") != null)
						{
							albumName = (String) node2.getProperty("ALBUMNAME");
						}
						else
						{
							albumName = "Unknown Album";
						}
						if(node3.getProperty("NAME") != null)
						{
							perfName = (String) node3.getProperty("NAME");
						}
						else
						{
							perfName = "Unknown Performer"; 
						}
						
						recsongsid.add(idsong);
						songs.add(idsong);
						
						rl = new RecommendationList(albumName,songName,perfName,idsong);
						recList.add(rl);
					}
					
					else if(genreIds.contains((node1.getProperty("GENREID").toString())) && countrec != 5 && !(recsongsid.contains(idsong)))
					{
						double zero = 0;
						//max = 0;
						res.instance(i).setValue(0, zero);
						countrec ++;
						//Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
						String songName = (String) node1.getProperty("NAME");
						
						Node node2 = (Node) indexAlbum.get("ID", node1.getProperty("ALBUMID").toString()).getSingle();
						Node node3 = (Node) indexPerformer.get("ID", node1.getProperty("PERFORMERID").toString()).getSingle();
						
						String albumName;
						String perfName;
						if(node2.getProperty("ALBUMNAME") != null)
						{
							albumName = (String) node2.getProperty("ALBUMNAME");
						}
						else
						{
							albumName = "Unknown Album";
						}
						if(node3.getProperty("NAME") != null)
						{
							perfName = (String) node3.getProperty("NAME");
						}
						else
						{
							perfName = "Unknown Performer"; 
						}
						
						recsongsid.add(idsong);
						songs.add(idsong);
						
						rl = new RecommendationList(albumName,songName,perfName,idsong);
						recList.add(rl);
					}
				}
			}
			
			
			for(int j = 0; j < (5 - countrec); j++)
			{
				String idsong = null;
				
				for(int i = 0; i < res.numInstances(); i++)
				{
					int idd = (int)res.instance(index).weight();
					idsong = Integer.toString(idd);
					if((res.instance(i).value(0) > max))
						{
						
						index = i;
						max = res.instance(i).value(0);
						}
				}
				//System.out.println("indexx " + index);
				double zero = 0;
				max = 0;
				res.instance(index).setValue(0, zero);

				//result = engine.execute("START me = node:Song(ID='" + idsong + "') MATCH (me) - [myrate:LISTEN] -> (song) WHERE ((myrate.ratingValue > 0.6) AND (myrate.numberofaction > 1)) RETURN distinct(song.ID),song.NAME, song.ALBUMID, song.PERFORMERID, myrate.timeofaction, myrate.frequency, myrate.numberofaction, myrate.ratingValue ORDER BY myrate.frequency LIMIT 5");
				Node node1 = (Node) indexSong.get("ID", idsong).getSingle();
				String songName = (String) node1.getProperty("NAME");
				
				Node node2 = (Node) indexAlbum.get("ID", node1.getProperty("ALBUMID").toString()).getSingle();
				Node node3 = (Node) indexPerformer.get("ID", node1.getProperty("PERFORMERID").toString()).getSingle();
				
				String albumName;
				String perfName;
				if(node2.getProperty("ALBUMNAME") != null)
				{
					albumName = (String) node2.getProperty("ALBUMNAME");
				}
				else
				{
					albumName = "Unknown Album";
				}
				if(node3.getProperty("NAME") != null)
				{
					perfName = (String) node3.getProperty("NAME");
				}
				else
				{
					perfName = "Unknown Performer"; 
				}
				
				recsongsid.add(idsong);
				songs.add(idsong);
				
				rl = new RecommendationList(albumName,songName,perfName,idsong);
				recList.add(rl);
			}
			
			tx.success();
			tx.close();
		
	}
	public void findHistory(String ID, ArrayList<RecommendationList> historyList)
	{
		
		int check;
		ExecutionEngine engine = new ExecutionEngine(graphDB);
		ExecutionResult result;
		Transaction tx = graphDB.beginTx();
		result = engine.execute("START me = node:User(userid='" + ID + "') MATCH (me)-[myrate:LISTEN]-> (song) RETURN song.ID, song.NAME, song.ALBUMID, song.PERFORMERID, myrate.frequency ORDER BY myrate.frequency DESC");
		int korayIterator=0;
		double[] array1 = new double[10000];
		for(Map<String, Object> row: result)
		{
			
			check = 0;
			String albumName = null;
			String perfName = null;
			String songName = null;
			String idsong = null;
			List<String> columns = result.columns();

					
			for (String column : columns) 
			{
				
				if(check == 2)
				{
					
					String namea = row.get(column).toString();
					Node node2 = (Node) indexAlbum.get("ID", namea).getSingle();
					albumName = (String) node2.getProperty("ALBUMNAME");
				}
				if(check == 0)
				{
					
					idsong = row.get(column).toString();

				}
				else if(check == 3)
				{
					
					String namep = row.get(column).toString();
					Node node3 = (Node) indexPerformer.get("ID", namep).getSingle();
					perfName = (String) node3.getProperty("NAME");
					RecommendationList element = new RecommendationList(albumName, songName, perfName,idsong);
					historyList.add(element);
				}
				else if(check == 1)
				{
					songName = row.get(column).toString();
				}
				else if(check == 4)
				{
					array1[korayIterator]=Double.parseDouble(row.get(column).toString());
					korayIterator++;
					//System.out.println("History fonksiyonunda frequency: "+row.get(column).toString());
				}
				check = check + 1;
					
			}

				
			}
	//	}
			
		
		
	}
	
	public static void registerShutdownHook( final GraphDatabaseService graphDb )
	{
		Runtime.getRuntime().addShutdownHook( new Thread()
	    {
	        @Override
	        public void run()
	        {
	            graphDb.shutdown();
	        }
	    } );

	}

	private void clearDb()
    {
        try
        {
            FileUtils.deleteRecursively( new File(workspacePath));//"C:\\Users\\isinsukatircioglu\\workspace\\RecommendationSystem\\db\\graphDB") );
        }
        catch ( IOException e )
        {
            throw new RuntimeException( e );
        }

    }

}
